﻿using System.Collections.Generic;
using Ozeki.VoIP;

namespace CallQueue
{
    class CallQueueExtension : IExtension
    {
        IExtensionContainer extensionContainer;
        List<CallQueueCallHandler> callHandlers;
        ICallManager callManager;
        List<string> members;
        object sync;

        public CallQueueExtension(string id, IExtensionContainer extensionContainer, ICallManager callManager, List<string> members)
        {
            sync = new object();
            callHandlers = new List<CallQueueCallHandler>();
            ExtensionID = id;
            this.extensionContainer = extensionContainer;
            this.callManager = callManager;
            this.members = members;
        }

        public string ExtensionID { get; private set; }

        public string Type
        {
            get { return "CallQueue"; }
        }

        public void OnCalled(IPBXCall call, IncomingCallParams callParams)
        {
            var callQueueHandler = new CallQueueCallHandler(call, callManager, extensionContainer, members);
            callQueueHandler.Completed += CallHandlerCompleted;
            callHandlers.Add(callQueueHandler);

            if (callHandlers.Count == 1)
            {
                callQueueHandler.Start();
            }
        }

        public void OnMessageReceived(Message message)
        {
        }

        void CallHandlerCompleted(object sender, System.EventArgs e)
        {
            lock (sync)
            {
                var callHandler = (CallQueueCallHandler)sender;

                if (callHandlers[0] == callHandler)
                {
                    callHandlers.Remove(callHandler);

                    if (callHandlers.Count >= 1)
                    {
                        var nextCallHandler = callHandlers[0];
                        nextCallHandler.Start();
                    }
                }
                else
                {
                    callHandlers.Remove(callHandler);
                }
            }
        }
    }
}
